package pusher

import (
	"context"
	config "gitea.timerzz.com/kedaya_haitao/pusher/kitex_gen/config"
	push "gitea.timerzz.com/kedaya_haitao/pusher/kitex_gen/push"

	"gitea.timerzz.com/kedaya_haitao/pusher/kitex_gen/push/pushservice"
	"github.com/cloudwego/kitex/client"
	"github.com/cloudwego/kitex/client/callopt"
)

type RPCClient interface {
	KitexClient() pushservice.Client
	Service() string
	Push(ctx context.Context, req *push.PushReq, callOptions ...callopt.Option) (r *push.Resp, err error)
	Add(ctx context.Context, req *config.PusherConfig, callOptions ...callopt.Option) (r *push.Resp, err error)
	List(ctx context.Context, req *push.ListPusherRequest, callOptions ...callopt.Option) (r *push.ListPusherResponse, err error)
	GetPusherOptions(ctx context.Context, callOptions ...callopt.Option) (r *push.GetPusherOptionsResponse, err error)
}

func NewRPCClient(dstService string, opts ...client.Option) (RPCClient, error) {
	kitexClient, err := pushservice.NewClient(dstService, opts...)
	if err != nil {
		return nil, err
	}
	cli := &clientImpl{
		service:     dstService,
		kitexClient: kitexClient,
	}

	return cli, nil
}

type clientImpl struct {
	service     string
	kitexClient pushservice.Client
}

func (c *clientImpl) Service() string {
	return c.service
}

func (c *clientImpl) KitexClient() pushservice.Client {
	return c.kitexClient
}

func (c *clientImpl) Push(ctx context.Context, req *push.PushReq, callOptions ...callopt.Option) (r *push.Resp, err error) {
	return c.kitexClient.Push(ctx, req, callOptions...)
}

func (c *clientImpl) Add(ctx context.Context, req *config.PusherConfig, callOptions ...callopt.Option) (r *push.Resp, err error) {
	return c.kitexClient.Add(ctx, req, callOptions...)
}

func (c *clientImpl) List(ctx context.Context, req *push.ListPusherRequest, callOptions ...callopt.Option) (r *push.ListPusherResponse, err error) {
	return c.kitexClient.List(ctx, req, callOptions...)
}

func (c *clientImpl) GetPusherOptions(ctx context.Context, callOptions ...callopt.Option) (r *push.GetPusherOptionsResponse, err error) {
	return c.kitexClient.GetPusherOptions(ctx, callOptions...)
}
